package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.bean.TrdVirtualOrderBean;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeTimeRequest;
import com.gic.spark.util.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.spark.api.java.JavaPairRDD;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.Optional;
import org.apache.spark.sql.Column;
import org.apache.spark.sql.Row;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.List;

/**
 * @description:
 * @author: wangxk
 * @date: 2020/8/10
 */
public class TagConsumeTimeFilter extends AbstractTagConsumRecordFilter{

    private static TagConsumeTimeFilter instance;

    public static TagConsumeTimeFilter getInstance() {
        if(null==instance){
            instance=new TagConsumeTimeFilter();
        }
        return instance;
    }

    private TagConsumeTimeFilter(){}

    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHiveOrder);
        result.add(dataSourceHiveOrderItem);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeTimeRequest consumeTimeRequest=(TagConsumeTimeRequest)request;
        JavaRDD<TrdVirtualOrderBean>consumeRecordRDD= MysqlRddManager.getPojoFromDataset(dataSourceHiveOrder.getDatasetByEntId(enterpriseId)
                                                        .filter(new Column("is_eff_order").equalTo(1)), TrdVirtualOrderBean.class).javaRDD();

        JavaRDD<Row>virtualOrderItemRdd=dataSourceHiveOrderItem.getDatasetByEntId(enterpriseId).select("virtual_order_id","ent_brand_id").javaRDD();

        JavaRDD<Tuple2<TrdVirtualOrderBean,Optional<Iterable<String>>>>orderRdd=consumeRecordRDD.mapToPair(data->Tuple2.apply(data.getVirtual_id(),data))
                                                                                .leftOuterJoin(virtualOrderItemRdd.mapToPair(row->Tuple2.apply(row.getLong(0),row.getString(1))).groupByKey())
                                                                                .map(data->data._2());

        consumeRecordRDD=statisticsTypeHandle(orderRdd,consumeTimeRequest);
        JavaRDD<Long>ecuRdd=consumeRecordRDD.filter(data-> StringUtils.isNotEmpty(data.getReceipts_time()))
                                                .mapToPair(data-> Tuple2.apply(data.getEcu_id(), DateUtil.strToDate(data.getReceipts_time(),DateUtil.FORMAT_DATETIME_19)))
                                                .filter(data->checkTime(consumeTimeRequest,data._2().getTime()))
                                                .reduceByKey((x,y)->x)
                                                .map(data->data._1());
        return ecuRdd;
    }
}
