package com.gic.spark.filter;

import com.alibaba.fastjson.JSONObject;
import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.bean.TrdVirtualOrderBean;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeChannelRequest;
import com.gic.spark.util.DateUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.Optional;
import org.apache.spark.sql.Row;
import org.json4s.jackson.Json;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.List;

/**
 * @description:
 * @author: wangxk
 * @date: 2020/8/10
 */
public class TagFirstConsumptionChannelFilter extends AbstractTagConsumRecordFilter{

    private static TagFirstConsumptionChannelFilter instance;

    public static TagFirstConsumptionChannelFilter getInstance() {
        if(null==instance){
            instance=new TagFirstConsumptionChannelFilter();
        }
        return instance;
    }
    private TagFirstConsumptionChannelFilter(){}

    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHiveOrder);
        result.add(dataSourceHiveOrderItem);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeChannelRequest channelRequest=(TagConsumeChannelRequest)request;
        JavaRDD<TrdVirtualOrderBean>consumeRecordRDD= MysqlRddManager.getPojoFromDataset(dataSourceHiveOrder.getDatasetByEntId(enterpriseId),TrdVirtualOrderBean.class).javaRDD();

        JavaRDD<Row>virtualOrderItemRdd=dataSourceHiveOrderItem.getDatasetByEntId(enterpriseId).select("virtual_order_id","ent_brand_id").javaRDD();

        JavaRDD<Tuple2<TrdVirtualOrderBean,Optional<Iterable<String>>>>orderRdd=consumeRecordRDD.mapToPair(data->Tuple2.apply(data.getVirtual_id(),data))
                                                                                .leftOuterJoin(virtualOrderItemRdd.mapToPair(row->Tuple2.apply(row.getLong(0),row.getString(1))).groupByKey())
                                                                                .map(data->data._2());

        consumeRecordRDD=statisticsTypeHandle(orderRdd,channelRequest);

        JavaRDD<Long>ecuRdd=consumeRecordRDD.filter(data-> StringUtils.isNotEmpty(data.getReceipts_time()))
                                .mapToPair(data-> Tuple2.apply(data.getEcu_id(),data))
                                .reduceByKey((x,y)->{
                                    if(DateUtil.strToDate(x.getReceipts_time(),DateUtil.FORMAT_DATETIME_19).getTime()
                                            <DateUtil.strToDate(y.getReceipts_time(),DateUtil.FORMAT_DATETIME_19).getTime()){
                                        return x;
                                    }else{
                                        return y;
                                    }
                                }).filter(data->{
                                    if(channelRequest.getChannelVals().contains(data._2().getOrder_channel_code())){
                                        return true;
                                    }
                                    return false;
                                }).map(data->data._1());
        return ecuRdd;
    }
}
