package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.bean.TrdVirtualOrderBean;
import com.gic.spark.entity.enumtype.YearMonthDayType;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeStoreRequest;
import com.gic.spark.util.DateUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.Optional;
import org.apache.spark.sql.Row;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.List;

/**
 * @description:
 * @author: wangxk
 * @date: 2020/8/11
 */
public class TagOfflineConsumptionStoreFilter extends AbstractTagConsumRecordFilter{

    private static TagOfflineConsumptionStoreFilter instance;

    public static TagOfflineConsumptionStoreFilter getInstance() {
        if(null==instance){
            instance=new TagOfflineConsumptionStoreFilter();
        }
        return instance;
    }

    private TagOfflineConsumptionStoreFilter(){}

    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHiveOrder);
        result.add(dataSourceHiveOrderItem);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeStoreRequest storeRequest=(TagConsumeStoreRequest)request;
        storeRequest.setYearMonthDayType(YearMonthDayType.DAY);
        JavaRDD<TrdVirtualOrderBean>consumeRecordRDD= MysqlRddManager.getPojoFromDataset(dataSourceHiveOrder.getDatasetByEntId(enterpriseId),TrdVirtualOrderBean.class).javaRDD();

        JavaRDD<Row>virtualOrderItemRdd=dataSourceHiveOrderItem.getDatasetByEntId(enterpriseId).select("virtual_order_id","ent_brand_id").javaRDD();

        JavaRDD<Tuple2<TrdVirtualOrderBean,Optional<Iterable<String>>>>orderRdd=consumeRecordRDD.mapToPair(data->Tuple2.apply(data.getVirtual_id(),data))
                                                                                .leftOuterJoin(virtualOrderItemRdd.mapToPair(row->Tuple2.apply(row.getLong(0),row.getString(1))).groupByKey())
                                                                                .map(data->data._2());

        consumeRecordRDD=statisticsTypeHandle(orderRdd,storeRequest);
        JavaRDD<Long>ecuRdd=consumeRecordRDD.filter(data->data.getOrder_channel_code()==1
                                                && null!=data.getStore_info_id()
                                                && StringUtils.isNotEmpty(data.getReceipts_time()))
                                                .filter(data->checkTime(storeRequest, DateUtil.strToDate(data.getReceipts_time(),DateUtil.FORMAT_DATETIME_19).getTime()))
                                                .mapToPair(data-> Tuple2.apply(data.getEcu_id(),data.getStore_info_id()))
                                                .filter(data->storeRequest.getStoreList().contains(String.valueOf(data._2())))
                                                .reduceByKey((x,y)->x)
                                                .map(data->data._1());
        return ecuRdd;
    }
}
